@@ -8,23 +8,26 @@ public API. It coordinates:
 - permutation generation (full matrix or streamed),
 - model evaluation (FastOLS or user-supplied stat_fn),
 - p-value and p-value CI calculation,
-- optional coefficient CI bounds/bands,
+- optional coefficient CI bounds/band,
 - packaging results into `RitestResult`.
 """
 
 from __future__ import annotations
 
 import os
-import warnings
 from concurrent.futures import ThreadPoolExecutor
 from typing import Callable, Dict, Optional, Tuple, cast
 
 import numpy as np
 import pandas as pd
 
-from .ci.coef_ci import Alt, coef_ci_band_fast, coef_ci_bounds_fast, coef_ci_bounds_generic
+from .ci.coef_ci import (
+    Alt,
+    coef_ci_band_fast,
+    coef_ci_bounds_fast,
+)
 from .ci.pvalue_ci import _PValCIMethod, pvalue_ci
-from .config import DEFAULTS
+from .config import DEFAULTS, _canonical_ci_method
 
 # FastOLS is the linear-path engine; NUMBA_OK tells us if its kernels are JITed.
 try:
@@ -55,16 +58,16 @@ def _coerce_n_jobs(val: int | None) -> int:
     return max(v, 1)
 
 
-def _coerce_ci_method(x: str | _PValCIMethod | None, fallback: _PValCIMethod) -> _PValCIMethod:
-    """Normalise user input to core labels `'cp'` or `'normal'`."""
+def _coerce_ci_method(
+    x: str | _PValCIMethod | None, fallback: _PValCIMethod
+) -> _PValCIMethod:
+    """Normalise user input to canonical labels."""
     if x is None:
         return fallback
-    s = str(x).strip().lower()
-    if s in {"cp", "clopper-pearson", "clopperpearson", "clopper", "exact"}:
-        return "cp"  # type: ignore[return-value]
-    if s in {"normal", "wald"}:
-        return "normal"  # type: ignore[return-value]
-    return fallback
+    try:
+        return _canonical_ci_method(x)  # type: ignore[return-value]
+    except ValueError:
+        return fallback
 
 
 def _bytes_per_row(n_obs: int, label_itemsize: int) -> int:
@@ -106,10 +109,9 @@ def ritest(  # noqa: C901
     seed: int | None = None,
     alpha: float | None = None,
     ci_method: str | _PValCIMethod | None = None,
-    ci_mode: str | None = None,  # "none" | "bounds" | "grid"
+    ci_mode: str | None = None,  # "none" | "bounds" | "band"
     ci_range: float | None = None,
     ci_step: float | None = None,
-    coef_ci_generic: bool | None = None,  # relevant only when stat_fn and grid
     n_jobs: int | None = None,
     # prebuilt permutations (matrix of permuted T labels)
     permutations: np.ndarray | None = None,
@@ -127,15 +129,15 @@ def ritest(  # noqa: C901
     reps = int(cfg["reps"]) if reps is None else int(reps)
     seed = int(cfg["seed"]) if seed is None else int(seed)
     alpha = float(cfg["alpha"]) if alpha is None else float(alpha)
-    ci_method = _coerce_ci_method(ci_method, fallback=cfg["ci_method"])
+    cfg_ci_method = _canonical_ci_method(cfg["ci_method"])
+    ci_method = _coerce_ci_method(ci_method, fallback=cfg_ci_method)
     ci_mode = str(cfg["ci_mode"]) if ci_mode is None else str(ci_mode)
     ci_range = float(cfg["ci_range"]) if ci_range is None else float(ci_range)
     ci_step = float(cfg["ci_step"]) if ci_step is None else float(ci_step)
     ci_tol = float(cfg["ci_tol"])  # reserved for future use
-    coef_ci_generic = (
-        bool(cfg["coef_ci_generic"]) if coef_ci_generic is None else bool(coef_ci_generic)
+    n_jobs = _coerce_n_jobs(
+        int(cfg.get("n_jobs", 1)) if n_jobs is None else int(n_jobs)
     )
-    n_jobs = _coerce_n_jobs(int(cfg.get("n_jobs", 1)) if n_jobs is None else int(n_jobs))
     # Memory/chunking knobs (soft budget)
     perm_chunk_bytes = int(cfg.get("perm_chunk_bytes", 256 * 1024 * 1024))
     perm_chunk_min_rows = int(cfg.get("perm_chunk_min_rows", 64))
@@ -159,14 +161,19 @@ def ritest(  # noqa: C901
         ci_range=ci_range,
         ci_step=ci_step,
         ci_tol=ci_tol,
-        coef_ci_generic=coef_ci_generic,
     )
 
     # 2) Observed statistic
     linear_model = v.stat_fn is None
+    need_coef_ci = linear_model and ci_mode != "none"
     if linear_model:
         ols = FastOLS(
-            v.y, v.X, v.treat_idx, weights=v.weights, cluster=v.cluster, compute_vcov=True
+            v.y,
+            v.X,
+            v.treat_idx,
+            weights=v.weights,
+            cluster=v.cluster,
+            compute_vcov=True,
         )
         obs_stat = float(ols.beta_hat)
         se_obs = float(ols.se)
@@ -179,29 +186,44 @@ def ritest(  # noqa: C901
         K_obs = float("nan")
         t_metric_lin = None  # type: ignore[assignment]
 
-    # Warn for potentially slow generic grid band when opted-in
-    if (not linear_model) and (ci_mode == "grid") and coef_ci_generic:
-        grid_size = max(int(round((2.0 * ci_range) / max(ci_step, 1e-12))) + 1, 1)
-        est_sec = float(v.warmup_time) * grid_size * reps
-        if est_sec > 10:
-            warnings.warn(
-                f"CI bands for stat_fn may take ~{est_sec:.1f} sec "
-                f"(warmup {v.warmup_time:.3f}s × grid≈{grid_size} × reps={reps})."
-            )
-
     # 3) Allocate outputs (small, O(reps))
     perm_stats = np.empty(reps, dtype=np.float64)
-    K_perm_local = np.empty(reps, dtype=np.float64) if linear_model else None
+    K_perm_local = np.empty(reps, dtype=np.float64) if need_coef_ci else None
 
     # Helper workers (capture `v`, `t_metric_lin` by closure)
-    def _fit_one(args: Tuple[int, np.ndarray]) -> Tuple[int, float, float]:
-        """Linear-path worker: (abs_index, T_perm_row) -> (abs_index, beta_r, Kr)."""
-        r_abs, T_perm = args
-        Xp = v.X.copy()
+    def _solve_linear_perm(
+        T_perm: np.ndarray,
+        X_template: np.ndarray | None = None,
+        *,
+        reuse_template: bool = False,
+    ) -> FastOLS:
+        """Solve linear model for a permuted treatment vector."""
+        if X_template is None:
+            Xp = v.X.copy()
+        elif reuse_template:
+            Xp = X_template
+        else:
+            Xp = X_template.copy()
         Xp[:, v.treat_idx] = T_perm  # int8 → float cast on assignment
-        ols_r = FastOLS(
-            v.y, Xp, v.treat_idx, weights=v.weights, cluster=v.cluster, compute_vcov=False
+        return FastOLS(
+            v.y,
+            Xp,
+            v.treat_idx,
+            weights=v.weights,
+            cluster=v.cluster,
+            compute_vcov=False,
         )
+
+    def _fit_one_beta(args: Tuple[int, np.ndarray]) -> Tuple[int, float]:
+        """Linear-path worker: returns only beta_r for ci_mode='none' cases."""
+        r_abs, T_perm = args
+        ols_r = _solve_linear_perm(T_perm)
+        return r_abs, float(ols_r.beta_hat)
+
+    def _fit_one_beta_K(args: Tuple[int, np.ndarray]) -> Tuple[int, float, float]:
+        """Linear-path worker: returns beta_r and Kr for CI computation."""
+        r_abs, T_perm = args
+        ols_r = _solve_linear_perm(T_perm)
         beta_r = float(ols_r.beta_hat)
         Kr = float(ols_r.c_vector @ t_metric_lin)  # type: ignore[arg-type]
         return r_abs, beta_r, Kr
@@ -225,31 +247,33 @@ def ritest(  # noqa: C901
             # Honor the supplied matrix size as the true reps
             reps = int(perms.shape[0])
             perm_stats = np.empty(reps, dtype=np.float64)
-            if linear_model:
+            if need_coef_ci:
                 K_perm_local = np.empty(reps, dtype=np.float64)
         # Process rows directly (no chunking for user-supplied matrix)
         if linear_model:
             if n_jobs == 1:
                 X_work = v.X.copy()
                 for r in range(reps):
-                    X_work[:, v.treat_idx] = perms[r]
-                    ols_r = FastOLS(
-                        v.y,
-                        X_work,
-                        v.treat_idx,
-                        weights=v.weights,
-                        cluster=v.cluster,
-                        compute_vcov=False,
+                    ols_r = _solve_linear_perm(
+                        perms[r], X_template=X_work, reuse_template=True
                     )
                     perm_stats[r] = float(ols_r.beta_hat)
-                    K_perm_local[r] = float(ols_r.c_vector @ t_metric_lin)  # type: ignore[index]
+                    if need_coef_ci:
+                        K_perm_local[r] = float(ols_r.c_vector @ t_metric_lin)  # type: ignore[index]
             else:
+                worker = _fit_one_beta_K if need_coef_ci else _fit_one_beta
                 with ThreadPoolExecutor(max_workers=n_jobs) as ex:
-                    for r_abs, beta_r, Kr in ex.map(
-                        lambda a: _fit_one(a), ((r, perms[r]) for r in range(reps))
-                    ):
-                        perm_stats[r_abs] = beta_r
-                        K_perm_local[r_abs] = Kr  # type: ignore[index]
+                    if need_coef_ci:
+                        for r_abs, beta_r, Kr in ex.map(
+                            worker, ((r, perms[r]) for r in range(reps))
+                        ):
+                            perm_stats[r_abs] = beta_r
+                            K_perm_local[r_abs] = Kr  # type: ignore[index]
+                    else:
+                        for r_abs, beta_r in ex.map(
+                            worker, ((r, perms[r]) for r in range(reps))
+                        ):
+                            perm_stats[r_abs] = beta_r
         else:
             if n_jobs == 1:
                 for r in range(reps):
@@ -278,24 +302,26 @@ def ritest(  # noqa: C901
                 if n_jobs == 1:
                     X_work = v.X.copy()
                     for r in range(reps):
-                        X_work[:, v.treat_idx] = T_perms[r]
-                        ols_r = FastOLS(
-                            v.y,
-                            X_work,
-                            v.treat_idx,
-                            weights=v.weights,
-                            cluster=v.cluster,
-                            compute_vcov=False,
+                        ols_r = _solve_linear_perm(
+                            T_perms[r], X_template=X_work, reuse_template=True
                         )
                         perm_stats[r] = float(ols_r.beta_hat)
-                        K_perm_local[r] = float(ols_r.c_vector @ t_metric_lin)  # type: ignore[index]
+                        if need_coef_ci:
+                            K_perm_local[r] = float(ols_r.c_vector @ t_metric_lin)  # type: ignore[index]
                 else:
+                    worker = _fit_one_beta_K if need_coef_ci else _fit_one_beta
                     with ThreadPoolExecutor(max_workers=n_jobs) as ex:
-                        for r, beta_r, Kr in ex.map(
-                            _fit_one, ((r, T_perms[r]) for r in range(reps))
-                        ):
-                            perm_stats[r] = beta_r
-                            K_perm_local[r] = Kr  # type: ignore[index]
+                        if need_coef_ci:
+                            for r, beta_r, Kr in ex.map(
+                                worker, ((r, T_perms[r]) for r in range(reps))
+                            ):
+                                perm_stats[r] = beta_r
+                                K_perm_local[r] = Kr  # type: ignore[index]
+                        else:
+                            for r, beta_r in ex.map(
+                                worker, ((r, T_perms[r]) for r in range(reps))
+                            ):
+                                perm_stats[r] = beta_r
             else:
                 if n_jobs == 1:
                     for r in range(reps):
@@ -304,7 +330,9 @@ def ritest(  # noqa: C901
                         perm_stats[r] = float(stat_fn_local(dfp))  # type: ignore[arg-type]
                 else:
                     with ThreadPoolExecutor(max_workers=n_jobs) as ex:
-                        for r, z in ex.map(_eval_one, ((r, T_perms[r]) for r in range(reps))):
+                        for r, z in ex.map(
+                            _eval_one, ((r, T_perms[r]) for r in range(reps))
+                        ):
                             perm_stats[r] = z
         else:
             # Streaming path: generate blocks with bounded memory and process each in turn.
@@ -313,7 +341,7 @@ def ritest(  # noqa: C901
             if chunk_rows <= 0:  # ultra-conservative fallback
                 chunk_rows = perm_chunk_min_rows
 
-            r0 = 0  # absolute write position into perm_stats / K_perm_local
+            r0 = 0  # absolute write position into perm_stats (and K_perm_local when present)
             if n_jobs == 1:
                 # Serial evaluation per block
                 X_work: Optional[np.ndarray] = None
@@ -332,17 +360,14 @@ def ritest(  # noqa: C901
                     if linear_model:
                         Xw = cast(np.ndarray, X_work)
                         for i in range(m):
-                            Xw[:, v.treat_idx] = block[i]
-                            ols_r = FastOLS(
-                                v.y,
-                                Xw,
-                                v.treat_idx,
-                                weights=v.weights,
-                                cluster=v.cluster,
-                                compute_vcov=False,
+                            ols_r = _solve_linear_perm(
+                                block[i], X_template=Xw, reuse_template=True
                             )
                             perm_stats[r0 + i] = float(ols_r.beta_hat)
-                            K_perm_local[r0 + i] = float(ols_r.c_vector @ t_metric_lin)  # type: ignore[index]
+                            if need_coef_ci:
+                                K_perm_local[r0 + i] = float(
+                                    ols_r.c_vector @ t_metric_lin
+                                )  # type: ignore[index]
                     else:
                         for i in range(m):
                             dfp = df.copy(deep=False)
@@ -363,11 +388,18 @@ def ritest(  # noqa: C901
                         m = block.shape[0]
                         if linear_model:
                             # Map absolute indices to rows in the current block
-                            for r_abs, beta_r, Kr in ex.map(
-                                _fit_one, ((r0 + i, block[i]) for i in range(m))
-                            ):
-                                perm_stats[r_abs] = beta_r
-                                K_perm_local[r_abs] = Kr  # type: ignore[index]
+                            worker = _fit_one_beta_K if need_coef_ci else _fit_one_beta
+                            if need_coef_ci:
+                                for r_abs, beta_r, Kr in ex.map(
+                                    worker, ((r0 + i, block[i]) for i in range(m))
+                                ):
+                                    perm_stats[r_abs] = beta_r
+                                    K_perm_local[r_abs] = Kr  # type: ignore[index]
+                            else:
+                                for r_abs, beta_r in ex.map(
+                                    worker, ((r0 + i, block[i]) for i in range(m))
+                                ):
+                                    perm_stats[r_abs] = beta_r
                         else:
                             for r_abs, z in ex.map(
                                 _eval_one, ((r0 + i, block[i]) for i in range(m))
@@ -390,69 +422,34 @@ def ritest(  # noqa: C901
     # 6) (Optionally) compute coefficient CI artifacts
     coef_ci_bounds: Optional[tuple[float, float]] = None
     coef_ci_band = None
-    band_valid_linear = linear_model  # True for linear path; False for generic
+    band_valid_linear = linear_model and need_coef_ci
 
-    if ci_mode != "none":
-        if linear_model:
-            if ci_mode in {"bounds", "grid"}:
-                coef_ci_bounds = coef_ci_bounds_fast(
-                    beta_obs=obs_stat,
-                    beta_perm=perm_stats,
-                    K_obs=K_obs,  # type: ignore[arg-type]
-                    K_perm=cast(np.ndarray, K_perm_local),  # type: ignore[arg-type]
-                    se=se_obs,
-                    alpha=alpha,
-                    ci_range=ci_range,
-                    ci_step=ci_step,
-                    alternative=alternative,
-                )
-            if ci_mode == "grid":
-                coef_ci_band = coef_ci_band_fast(
-                    beta_obs=obs_stat,
-                    beta_perm=perm_stats,
-                    K_obs=K_obs,  # type: ignore[arg-type]
-                    K_perm=cast(np.ndarray, K_perm_local),  # type: ignore[arg-type]
-                    se=se_obs,
-                    ci_range=ci_range,
-                    ci_step=ci_step,
-                    alternative=alternative,
-                )
-        else:
-            if coef_ci_generic:
-
-                def _runner(beta0: float) -> float:
-                    shifted = obs_stat - beta0
-                    if alternative == "two-sided":
-                        return (np.abs(perm_stats - beta0) >= abs(shifted)).mean()
-                    elif alternative == "right":
-                        return (perm_stats - beta0 >= shifted).mean()
-                    else:
-                        return (perm_stats - beta0 <= shifted).mean()
-
-                se_scale = float(np.nanstd(perm_stats, ddof=1)) if reps >= 2 else 1.0
-                if not (se_scale > 0.0 and np.isfinite(se_scale)):
-                    se_scale = 1.0
-
-                if ci_mode in {"bounds", "grid"}:
-                    coef_ci_bounds = coef_ci_bounds_generic(
-                        obs_stat,
-                        runner=_runner,
-                        alpha=alpha,
-                        ci_range=ci_range,
-                        ci_step=ci_step,
-                        se=se_scale,
-                        alternative=alternative,
-                    )
-                if ci_mode == "grid":
-                    grid = (
-                        np.arange(
-                            -ci_range * se_scale, ci_range * se_scale + 1e-12, ci_step * se_scale
-                        )
-                        + obs_stat
-                    )
-                    pvals = np.array([_runner(b0) for b0 in grid], dtype=np.float64)
-                    coef_ci_band = (grid, pvals)
-                    band_valid_linear = False
+    if need_coef_ci:
+        K_perm_arr = cast(np.ndarray, K_perm_local)
+        if ci_mode in {"bounds", "band"}:
+            coef_ci_bounds = coef_ci_bounds_fast(
+                beta_obs=obs_stat,
+                beta_perm=perm_stats,
+                K_obs=K_obs,  # type: ignore[arg-type]
+                K_perm=K_perm_arr,
+                se=se_obs,
+                alpha=alpha,
+                ci_range=ci_range,
+                ci_step=ci_step,
+                alternative=alternative,
+            )
+        if ci_mode == "band":
+            coef_ci_band = coef_ci_band_fast(
+                beta_obs=obs_stat,
+                beta_perm=perm_stats,
+                K_obs=K_obs,  # type: ignore[arg-type]
+                K_perm=K_perm_arr,
+                se=se_obs,
+                ci_range=ci_range,
+                ci_step=ci_step,
+                alternative=alternative,
+            )
+            band_valid_linear = True
 
     # 7) STRICT GATING of outputs before building result
     _bounds = coef_ci_bounds
@@ -465,16 +462,21 @@ def ritest(  # noqa: C901
         _band_valid_linear = False
     elif ci_mode == "bounds":
         _band = None
-        # KEEP the linear flag: True for linear, False for generic
-    elif ci_mode == "grid":
-        _bounds = None
-        if (stat_fn is not None) and (not coef_ci_generic):
-            _band = None
-            _band_valid_linear = False
+    elif ci_mode == "band":
+        pass
     else:
         raise ValueError(f"Unknown ci_mode: {ci_mode!r}")
 
     # 8) Package results
+    strata_count = (
+        int(np.unique(v.strata).size) if v.has_strata and v.strata is not None else None
+    )
+    cluster_count = (
+        int(np.unique(v.cluster).size)
+        if v.has_cluster and v.cluster is not None
+        else None
+    )
+
     settings: Dict[str, object] = {
         "alpha": alpha,
         "seed": seed,
@@ -485,7 +487,8 @@ def ritest(  # noqa: C901
         "ci_step": ci_step,
         "alternative": alternative,
         "n_jobs": n_jobs,
-        "coef_ci_generic": coef_ci_generic,
+        "strata": strata_count,
+        "clusters": cluster_count,
     }
 
     res = RitestResult(
